# -*- coding: utf-8 -*-
# Copyright (C) 2020 Adrien Delle Cave
# SPDX-License-Identifier: GPL-3.0-or-later
"""xtrargon_ng.classes.abstract.modules"""

import abc
import logging
import threading

from dwho.classes.modules import DWhoModuleBase

from .db import XtrargonDBBase
from ..lock_db import BackupsLockDB


LOG           = logging.getLogger('xtrargon_ng.classes.abstract.modules')

DB_URI_PARAMS = (('autocommit', '1'),
                 ('charset', 'utf8'),
                 ('cursorclass', 'Dict'),
                 ('wait_timeout', '2147483'))


class XtrargonDBModuleBase(XtrargonDBBase, DWhoModuleBase):
    __metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def MODULE_NAME(self):
        return

    def __init__(self):
        XtrargonDBBase.__init__(self)
        DWhoModuleBase.__init__(self)

        self._db_lock  = None

        self.lock_db   = None
        self.db_locked = None

    # pylint: disable-msg=attribute-defined-outside-init
    def _init_db(self, module = None, default_cnf_file = 'client.cnf', host = None, port = None):
        self.lock_db   = threading.Event()
        self.db_locked = threading.Event()

        XtrargonDBBase._init_db(self, module, default_cnf_file, host, port)

    def _init_lock_db(self, config):
        if self.dbconf and self.dburi:
            self._check_db_server(self.dburi)
            if self._db_lock:
                self._terminate_lock_db()
            self._db_lock = BackupsLockDB(self, config)
            self._db_lock.daemon = True

    def _start_lock_db(self):
        if self._db_lock:
            self._db_lock.start()

    def _terminate_lock_db(self):
        if self._db_lock:
            self._db_lock.terminate()
            self._db_lock = None

    def _clear_lock_db(self):
        if self.lock_db:
            self.lock_db.clear()
        if self.db_locked:
            self.db_locked.clear()
